#! /bin/bash

PROTOCOL_VERSION='3.0.0'


function process {
  ############ INFO #################################################################
  #
  # this script must get samba.tar.gz from standard input eg. cat samba.tar.gz | sudo ./process-samba.sh
  #
  ###################################################################################

  ############ INITIALIZATION #######################################################
  #
  # set variables and call service init script
  #

  [ -r "${WORK_DIR}/fac_settings" ] || internal_error E_READ_FILE   # check if facility settings are readable
  [ -r "${WORK_DIR}/fac_paths" ] || internal_error E_READ_FILE      # check if facility paths are readable

  IFS=$':'
  read REWRITE_CONF SMBCONF_PATH < "${WORK_DIR}/fac_settings"      # read facility settings from file
  unset IFS

  ###################################################################################

  ############# ERROR DEFINITIONS ###################################################
  E_READ_FILE="Unable to read some of important config files."
  E_READ_FILE_NO=1
  E_CREATE_PATH="Unable to create paths for share."
  E_CREATE_PATH_NO=2
  E_PERMISSION_PATH="Unable to change permissions for share on it's folders."
  E_PERMISSION_PATH_NO=3
  E_SHARE_DEF_FILE="Unable to copy(replace) smb.perun.conf(.new) in Samba config folder."
  E_SHARE_DEF_FILE_NO=4
  E_SMB_CONF_WRITE="smb.conf file is not writeable by slave script."
  E_SMB_CONF_WRITE_NO=5
  E_USER_DEF_FILE="Unable to copy(replace) smbusers.perun.map(.new) in Samba config folder."
  E_USER_DEF_FILE_NO=6
  E_SMBCONF_UPDATE="Unable to update refference to username map file in smb.conf"
  E_SMBCONF_UPDATE_NO=7
  E_UNIX_USER_NOT_EXIST="Unable to create Samba user (not exists on machine - repeat passwd propagation ?)"
  E_UNIX_USER_NOT_EXIST_NO=8
  E_CREATE_SMB_USER="Unable to create Samba user (privileges to run smbpasswd ?)"
  E_CREATE_SMB_USER_NO=9

  ###################################################################################

  ############# CHECK FILES #########################################################
  #
  # check for existence and readability of important files
  #

  [ -r "${WORK_DIR}/smb.perun.conf" ] || internal_error E_READ_FILE
  [ -r "${WORK_DIR}/smb_users" ] || internal_error E_READ_FILE
  [ -r "$SMBCONF_PATH/smb.conf" ] || internal_error E_READ_FILE

  ############# CREATE PATHS ########################################################
  #
  # create proper paths (already existing paths are only changed with chown and chmod
  # to respect perun privileges)
  #

  IFS=$':'
  while read S_GROUP S_PATH
  do
   if [ ! -d "$S_PATH" ]; then
    mkdir -p "$S_PATH" || internal_error E_CREATE_PATH
   fi
   chown :$S_GROUP $S_PATH || internal_error E_PERMISSION_PATH
   chmod 771 "$S_PATH" || internal_error E_PERMISSION_PATH
  done < "${WORK_DIR}/fac_paths"
  unset IFS

  ###################################################################################

  ############# CREATE SHARES DEFINITION ############################################
  #
  # copy share definition to Samba config folder
  # if rewrite is YES, overwrite previous file and make refference in smb.conf
  #

  # REWRITE_CONF = NO
  if [ "$REWRITE_CONF" == "0" ]; then

   # copy share definitions into samba conf folder (replace old .new perun file)
   cp -f "${WORK_DIR}/smb.perun.conf" "$SMBCONF_PATH/smb.perun.conf.new" || internal_error E_SHARE_DEF_FILE

  fi

  # REWRITE_CONF = YES
  if [ "$REWRITE_CONF" == "1" ]; then

   #check if smb.conf is writeable
   [ -w "$SMBCONF_PATH/smb.conf" ] || internal_error E_SMB_CONF_WRITE

   # copy new definitions into samba conf folder (replace old perun file)
   cp -f "${WORK_DIR}/smb.perun.conf" "$SMBCONF_PATH/smb.perun.conf" || internal_error E_SHARE_DEF_FILE

   # search for proper Perun refference in smb.conf file 
   grep "include = $SMBCONF_PATH/smb.perun.conf" "$SMBCONF_PATH/smb.conf" > /dev/null 
   if [ $? -gt 0 ]; then
    # refference not found - append one at the end of smb.conf
    echo "include = $SMBCONF_PATH/smb.perun.conf" >> "$SMBCONF_PATH/smb.conf"
   fi

  fi

  ###################################################################################

  ############# CREATE USERNAME MAP FILE ############################################
  #
  # create smbusers.perun.map file containting map o users "unix_user = samba_user"
  # copy it to Samba config folder
  # if rewrite is YES, overwrite previous file and make refference in smb.conf
  # 

  # read source file and make it smbusers like
  IFS=$':'
  while read SMB_NAME UNIX_NAME PASS
  do
   echo "$UNIX_NAME = $SMB_NAME" >> "${WORK_DIR}/smbusers.perun.map"
  done < "${WORK_DIR}/smb_users"
  unset IFS

  # REWRITE_CONF = NO
  if [ "$REWRITE_CONF" == "0" ]; then

   # copy share definitions into samba conf folder (replace old .new perun file)
   cp -f "${WORK_DIR}/smbusers.perun.map" "$SMBCONF_PATH/smbusers.perun.map.new" || internal_error E_USER_DEF_FILE

  fi

  # REWRITE_CONF = YES
  if [ "$REWRITE_CONF" == "1" ]; then

   # check if smb.conf is writeable
   [ -w "$SMBCONF_PATH/smb.conf" ] || internal_error E_SMB_CONF_WRITE

   # copy new username map definitions into samba conf folder (replace old perun file)
   cp -f "${WORK_DIR}/smbusers.perun.map" "$SMBCONF_PATH/smbusers.perun.map" || internal_error E_USER_DEF_FILE

   # check smb.conf for any username map refference 
   grep "username map = "* "$SMBCONF_PATH/smb.conf" > /dev/null 
   if [ $? -eq 0 ]; then
    # IF THERE IS ANY REFFERENCE, make sure to point all of them to smbusers.perun.map to avoid Samba to have different username map files 
    sed -i "s,^username map.*,username map = $SMBCONF_PATH/smbusers.perun.map,g" "$SMBCONF_PATH/smb.conf" > /dev/null || internal_error E_SMBCONF_UPDATE
   else
    # IF THERE IS NOT REFFERENCE, make sure to add one at [global] section => first line
    sed -i "1 i username map = $SMBCONF_PATH/smbusers.perun.map" "$SMBCONF_PATH/smb.conf" > /dev/null || internal_error E_SMBCONF_UPDATE
   fi

  fi

  ###################################################################################


  ############# CREATE USERS IN SAMBA ###############################################
  #
  # Check if user's unix_name exists and creates him/her in Samba database
  # This is just test dummy - all users have the same password "1234"
  # This code will be completly removed after Moonshot+Samba development is finished
  #

  # extract users from smb_users file
  IFS=$':'
  while read SMB_NAME UNIX_NAME PASS
  do
   # check if user exist on system
   id $UNIX_NAME > /dev/null || internal_error E_UNIX_USER_NOT_EXIST
   
   # check if user exists in samba (by enabling him/her)
   smbpasswd -s -e $UNIX_NAME &>> /dev/null
   # if user does'not exists, try to create one
    if [ $? -gt 0 ]; then
     (echo $PASS; echo $PASS) | smbpasswd -s -a $UNIX_NAME > /dev/null || internal_error E_CREATE_SMB_USER
    fi
  done < "${WORK_DIR}/smb_users"
  unset IFS

  ##################################################################################
}
